import { HttpOptions, Body, BodyType, ResponseType, PartialOptions } from './types/http';
/**
 * makes a HTTP request
 *
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function request<T>(options: HttpOptions): Promise<T>;
/**
 * makes a GET request
 *
 * @param url request URL
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function get<T>(url: string, options: PartialOptions): Promise<T>;
/**
 * makes a POST request
 *
 * @param url request URL
 * @param body request body
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function post<T>(url: string, body: Body, options: PartialOptions): Promise<T>;
/**
 * makes a PUT request
 *
 * @param url request URL
 * @param body request body
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function put<T>(url: string, body: Body, options: PartialOptions): Promise<T>;
/**
 * makes a PATCH request
 *
 * @param url request URL
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function patch<T>(url: string, options: PartialOptions): Promise<T>;
/**
 * makes a DELETE request
 *
 * @param url request URL
 * @param options request options
 *
 * @return promise resolving to the response
 */
declare function deleteRequest<T>(url: string, options: PartialOptions): Promise<T>;
declare const _default: {
    request: typeof request;
    get: typeof get;
    post: typeof post;
    put: typeof put;
    patch: typeof patch;
    delete: typeof deleteRequest;
    ResponseType: typeof ResponseType;
    BodyType: typeof BodyType;
};
export default _default;
